@@ -1,7 +1,7 @@
 /*
  * isd_server.cpp - ISD Server
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *  
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -29,14 +29,14 @@
 #include <QtCore/QTemporaryFile>
 #include <QtCore/QTimer>
 #include <QtGui/QMessageBox>
+#include <QtGui/QPushButton>
 #include <QtNetwork/QHostInfo>
 #include <QtNetwork/QTcpSocket>
 
 #include "isd_server.h"
 #include "isd_connection.h"
 #include "dsa_key.h"
-#include "local_system.h"
-#include "remote_control_widget.h"
+#include "local_system_ica.h"
 #include "ivs.h"
 #include "lock_widget.h"
 #include "messagebox.h"
@@ -47,18 +47,16 @@
 
 static isdServer * __isd_server = NULL;
 
-QByteArray isdServer::s_appInternalChallenge;
 QStringList isdServer::s_allowedDemoClients;
 
 
 
 isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 	QTcpServer(),
-	m_ivs( new IVS( _ivs_port, _argc, _argv ) ),
+	m_ivs( NULL ),
 	m_demoClient( NULL ),
 	m_demoServer( NULL ),
-	m_lockWidget( NULL ),
-	m_remoteControlWidget( NULL )
+	m_lockWidget( NULL )
 {
 	if( __isd_server ||
 			listen( QHostAddress::LocalHost, __isd_port ) == FALSE )
@@ -66,7 +64,7 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 		// uh oh, already an ISD running or port isn't available...
 		qCritical( "isdServer::isdServer(...): "
 				"could not start ISD server: %s",
-					errorString().toAscii().constData() );
+					errorString().toUtf8().constData() );
 		messageBox::trySysTrayMessage( tr( "ISD-server error" ),
 			tr( "The ISD-server could not be started because "
 				"port %1 is already in use. Please make sure "
@@ -76,8 +74,6 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 							messageBox::Critical );
 	}
 
-	m_ivs->start(/* QThread::HighPriority*/ );
-
 	connect( this, SIGNAL( newConnection() ),
 			this, SLOT( acceptNewConnection() ) );
 
@@ -91,6 +87,9 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 
 	// finally we set the global pointer to ourself
 	__isd_server = this;
+
+	m_ivs = new IVS( _ivs_port, _argc, _argv );
+	m_ivs->start(/* QThread::HighPriority*/ );
 }
 
 
@@ -147,15 +146,15 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 			const QString cmds = msg_in.arg( "cmds" ).toString();
 			if( !cmds.isEmpty() )
 			{
-				QProcess::execute( cmds );
+				QProcess::startDetached( cmds );
 			}
 			break;
 		}
 
 		case ISD::StartFullScreenDemo:
 		case ISD::StartWindowDemo:
-		case ISD::ViewRemoteDisplay:
-		case ISD::RemoteControlDisplay:
+/*		case ISD::ViewRemoteDisplay:
+		case ISD::RemoteControlDisplay:*/
 			action = msg_in.arg( "ip" ).toString();
 			break;
 
@@ -195,6 +194,11 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 			localSystem::reboot();
 			break;
 
+		case ISD::DisableLocalInputs:
+			localSystem::disableLocalInputs(
+					msg_in.arg( "disabled" ).toBool() );
+			break;
+
 		case ISD::SetRole:
 		{
 			const int role = msg_in.arg( "role" ).toInt();
@@ -317,7 +321,7 @@ bool isdServer::protocolInitialization( socketDevice & _sd,
 	_sd.read( (char *) &chosen, sizeof( chosen ) );
 
 	const int MAX_HOST_LEN = 255;
-	char host[MAX_HOST_LEN];
+	char host[MAX_HOST_LEN+1];
 	_sd.sockDispatcher()( host, MAX_HOST_LEN, SocketGetIPBoundTo,
 								_sd.user() );
 	host[MAX_HOST_LEN] = 0;
@@ -350,7 +354,7 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 {
 	// find out IP of host - needed at several places
 	const int MAX_HOST_LEN = 255;
-	char host[MAX_HOST_LEN];
+	char host[MAX_HOST_LEN+1];
 	_sd( host, MAX_HOST_LEN, SocketGetIPBoundTo, _user );
 	host[MAX_HOST_LEN] = 0;
 	static QStringList __denied_hosts, __allowed_hosts;
@@ -389,10 +393,23 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 		// host has to be in list of allowed hosts
 		case ItalcAuthHostBased:
 		{
+			if( s_allowedDemoClients.isEmpty() )
+			{
+				break;
+			}
+			QStringList allowed;
+			foreach( const QString a, s_allowedDemoClients )
+			{
+				const QString h = a.split( ':' )[0];
+				if( !allowed.contains( h ) )
+				{
+					allowed.push_back( h );
+				}
+			}
 			// already valid IP?
 			if( QHostAddress().setAddress( host ) )
 			{
-				if( s_allowedDemoClients.contains( host ) )
+				if( allowed.contains( host ) )
 				{
 					result = ItalcAuthOK;
 				}
@@ -408,8 +425,8 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 				// client-list
 				foreach( const QHostAddress a, addr )
 				{
-					if( s_allowedDemoClients.contains(
-								a.toString() ) )
+	if( allowed.contains( a.toString() ) ||
+			a.toString() == QHostAddress( QHostAddress::LocalHost ).toString() )
 					{
 						result = ItalcAuthOK;
 						break;
@@ -491,12 +508,12 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 		case ItalcAuthAppInternalChallenge:
 		{
 			// generate challenge
-			s_appInternalChallenge = dsaKey::generateChallenge();
+			__appInternalChallenge = dsaKey::generateChallenge();
 			sdev.write( QVariant() );
 			// is our client able to read this byte-array? if so,
 			// it's for sure running inside the same app
 			result = ( sdev.read().toByteArray() ==
-						s_appInternalChallenge ) ?
+						__appInternalChallenge ) ?
 						ItalcAuthOK : ItalcAuthFailed;
 			break;
 		}
@@ -511,6 +528,7 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 							QFile::WriteOwner );
 			tf.open();
 			tf.write( chall );
+			tf.flush();
 			sdev.write( tf.fileName() );
 			// is our client able to read the file? if so,
 			// it's running as the same user as this piece of
@@ -541,53 +559,35 @@ quint16 isdServer::isdPort( void )
 
 
 
+
 isdServer::accessDialogResult isdServer::showAccessDialog(
 							const QString & _host )
 {
-	QMessageBox m(
-#ifdef QMESSAGEBOX_EXT_SUPPORT
-			QMessageBox::Question,
-#endif
+	QMessageBox m( QMessageBox::Question,
 			tr( "Confirm access" ),
 			tr( "Somebody at host %1 tries to access your screen. "
 				"Do you want to grant him/her access?" ).
 								arg( _host ),
-#ifdef QMESSAGEBOX_EXT_SUPPORT
-				QMessageBox::Yes | QMessageBox::No
-#else
-			QMessageBox::Question,
-			QMessageBox::Yes,
-			QMessageBox::No | QMessageBox::Escape,
-			QMessageBox::NoToAll | QMessageBox::Default
-#endif
-			);
-#ifdef QMESSAGEBOX_EXT_SUPPORT
+				QMessageBox::Yes | QMessageBox::No );
+
 	QPushButton * never_btn = m.addButton( tr( "Never for this session" ),
 							QMessageBox::NoRole );
 	QPushButton * always_btn = m.addButton( tr( "Always for this session" ),
 							QMessageBox::YesRole );
 	m.setDefaultButton( never_btn );
 	m.setEscapeButton( m.button( QMessageBox::No ) );
-#else
-	m.setButtonText( 2, tr( "Never for this session" ) );
-#endif
+
 	localSystem::activateWindow( &m );
 
 	const int res = m.exec();
-#ifdef QMESSAGEBOX_EXT_SUPPORT
 	if( m.clickedButton() == never_btn )
-#else
-	if( res == QMessageBox::NoToAll )
-#endif
 	{
 		return( Never );
 	}
-#ifdef QMESSAGEBOX_EXT_SUPPORT
 	else if( m.clickedButton() == always_btn )
 	{
 		return( Always );
 	}
-#endif
 	else if( res == QMessageBox::No )
 	{
 		return( No );
@@ -597,6 +597,7 @@ isdServer::accessDialogResult isdServer::showAccessDialog(
 
 
 
+
 void isdServer::acceptNewConnection( void )
 {
 	QTcpSocket * sock = nextPendingConnection();
@@ -684,12 +685,12 @@ void isdServer::checkForPendingActions( void )
 				displayTextMessage( data );
 				break;
 
-			case ISD::ViewRemoteDisplay:
+/*			case ISD::ViewRemoteDisplay:
 			case ISD::RemoteControlDisplay:
 				remoteControlDisplay( data,
 		( m_pendingActions.front().first == ISD::ViewRemoteDisplay ) );
 				break;
-
+*/
 			default:
 				qWarning( "isdServer::checkForPendingActions():"
 						" unhandled command %d",
@@ -711,27 +712,10 @@ void isdServer::demoWindowClosed( QObject * )
 
 
 
-void isdServer::remoteControlWidgetClosed( QObject * )
-{
-	m_remoteControlWidget = NULL;
-}
-
-
-
-
 void isdServer::startDemo( const QString & _master_host, bool _fullscreen )
 {
 	delete m_demoClient;
 	m_demoClient = NULL;
-/*	if( m_demoServer && _master_host.section( ':', 0, 0 ) ==
-				m_demoServer->serverAddress().toString() &&
-		_master_host.section( ':', 1, 1 ).toInt() ==
-						m_demoServer->serverPort() )
-	{
-		// somehow we tried to start a demo on demo-server which makes
-		// no sense and results in a deadlock...
-		return;
-	}*/
 	// if a demo-server is started, it's likely that the demo was started
 	// on master-computer as well therefore we deny starting a demo on
 	// hosts on which a demo-server is running
@@ -788,24 +772,12 @@ void isdServer::displayTextMessage( const QString & _msg )
 
 
 
-void isdServer::remoteControlDisplay( const QString & _ip, bool _view_only )
-{
-	if( m_remoteControlWidget  )
-	{
-		return;
-	}
-	m_remoteControlWidget = new remoteControlWidget( _ip, _view_only );
-	connect( m_remoteControlWidget, SIGNAL( destroyed( QObject * ) ),
-			this, SLOT( remoteControlWidgetClosed( QObject * ) ) );
-}
-
-
-
-
 void isdServer::allowDemoClient( const QString & _host )
 {
+	const QString h = _host.split( ':' )[0];
+	const QString p = _host.contains( ':' ) ? ':'+_host.split( ':' )[1] : "";
 	// already valid IP?
-	if( QHostAddress().setAddress( _host ) )
+	if( QHostAddress().setAddress( h ) )
 	{
 		if( !s_allowedDemoClients.contains( _host ) )
 		{
@@ -814,13 +786,13 @@ void isdServer::allowDemoClient( const QString & _host )
 		return;
 	}
 	foreach( const QHostAddress a,
-				QHostInfo::fromName( _host ).addresses() )
+				QHostInfo::fromName( h ).addresses() )
 	{
-		const QString h = a.toString();
-			if( !s_allowedDemoClients.contains( h ) )
-			{
-				s_allowedDemoClients.push_back( h );
-			}
+		const QString h2 = a.toString();
+		if( !s_allowedDemoClients.contains( h2+p ) )
+		{
+			s_allowedDemoClients.push_back( h2+p );
+		}
 	}
 }
 
@@ -829,16 +801,18 @@ void isdServer::allowDemoClient( const QString & _host )
 
 void isdServer::denyDemoClient( const QString & _host )
 {
+	const QString h = _host.split( ':' )[0];
+	const QString p = _host.contains( ':' ) ? ':'+_host.split( ':' )[1] : "";
 	// already valid IP?
-	if( QHostAddress().setAddress( _host ) )
+	if( QHostAddress().setAddress( h ) )
 	{
 		s_allowedDemoClients.removeAll( _host );
 		return;
 	}
 	foreach( const QHostAddress a,
-				QHostInfo::fromName( _host ).addresses() )
+				QHostInfo::fromName( h ).addresses() )
 	{
-		s_allowedDemoClients.removeAll( a.toString() );
+		s_allowedDemoClients.removeAll( a.toString()+p );
 	}
 }
 
@@ -848,9 +822,9 @@ void isdServer::denyDemoClient( const QString & _host )
 void isdServer::errorMsgAuth( const QString & _ip )
 {
 	messageBox::trySysTrayMessage( tr( "Authentication error" ),
-		tr( "Somebody (IP: %1) tried to access this computer "
-			"but could not authenticate itself "
-			"successfully!" ).arg( QString( _ip ) ),
+			tr( "Somebody (IP: %1) tried to access this computer "
+					"but could not authenticate itself "
+					"successfully!" ).arg( QString( _ip ) ),
 						messageBox::Critical );
 }
 
@@ -981,12 +955,12 @@ public:
 					cmd == ISD::StartFullScreenDemo );
 				break;
 
-			case ISD::ViewRemoteDisplay:
+/*			case ISD::ViewRemoteDisplay:
 			case ISD::RemoteControlDisplay:
 				remoteControlDisplay( msg_in.arg( "ip" ).
 								toString(),
 						cmd == ISD::ViewRemoteDisplay );
-				break;
+				break;*/
 
 			case ISD::DisplayTextMessage:
 				displayTextMessage( msg_in.arg( "msg" ).
