@@ -1,7 +1,7 @@
 /*
  * classroom_manager.cpp - implementation of classroom-manager
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -58,6 +58,9 @@
 #include "tool_button.h"
 #include "messagebox.h"
 
+#define DEFAULT_WINDOW_WIDTH	1005
+#define DEFAULT_WINDOW_HEIGHT	700
+
 
 template<typename T>
 inline T roundCorrect( T _val )
@@ -81,7 +84,8 @@ QPixmap * classRoomItem::s_clientObservedPixmap = NULL;
 
 
 
-classroomManager::classroomManager( mainWindow * _main_window, QWidget * _parent ) :
+classroomManager::classroomManager( mainWindow * _main_window,
+							QWidget * _parent ) :
 	sideBarWidget( QPixmap( ":/resources/classroom_manager.png" ),
 			tr( "Classroom-Manager" ),
 			tr( "Use this workspace to manage your computers and "
@@ -90,6 +94,7 @@ classroomManager::classroomManager( mainWindow * _main_window, QWidget * _parent
 	m_personalConfiguration( localSystem::personalConfigPath() ),
 	m_globalClientConfiguration( localSystem::globalConfigPath() ),
 	m_quickSwitchMenu( new QMenu( this ) ),
+	m_qsmClassRoomSeparator( m_quickSwitchMenu->addSeparator() ),
 	m_globalClientMode( client::Mode_Overview ),
 	m_clientUpdateInterval( 1 )
 {
@@ -105,17 +110,16 @@ classroomManager::classroomManager( mainWindow * _main_window, QWidget * _parent
 					"in this list." ) );
 
 	QStringList columns;
-	columns << tr( "Classrooms/computers" ) << tr( "IP-address" );
+	columns << tr( "Classrooms/computers" ) << tr( "IP-address" ) << tr( "Usernames" );
 	m_view->setHeaderLabels( columns );
+	m_view->hideColumn( 2 );
 	m_view->setSelectionMode( QTreeWidget::ExtendedSelection );
 	m_view->setIconSize( QSize( 22, 22 ) );
 	m_view->header()->resizeSection( m_view->header()->logicalIndex( 0 ),
 									200 );
-/*	m_view->setTreeStepSize( 12 );
-	m_view->setDefaultRenameAction( QTreeWidget::Accept );
-	m_view->setSorting( 0 );*/
 	m_view->setSortingEnabled( TRUE );
 	m_view->setRootIsDecorated( TRUE );
+	m_view->sortItems( 0, Qt::AscendingOrder );
 //	m_view->setShowToolTips( TRUE );
 	m_view->setWindowTitle( tr( "Classroom-Manager" ) );
 	m_view->setContextMenuPolicy( Qt::CustomContextMenu );
@@ -127,10 +131,18 @@ classroomManager::classroomManager( mainWindow * _main_window, QWidget * _parent
 
 	QFont f;
 	f.setPixelSize( 12 );
+
+	m_showUserColumnCheckBox = new QCheckBox( tr( "Show usernames" ),
+			contentParent() );
+	m_showUserColumnCheckBox->setFont( f );
+	contentParent()->layout()->addWidget( m_showUserColumnCheckBox );
+	connect( m_showUserColumnCheckBox, SIGNAL( stateChanged( int ) ),
+			this, SLOT( showUserColumn( int ) ) );
+
 	QLabel * help_txt = new QLabel(
 		tr( "\nUse the context-menu (right mouse-button) to add/remove "
 			"computers and/or classrooms.\n\n"
-			"Once you did so you can show or hide clients by "
+			"Once you did so you can show or hide computers by "
 			"double-clicking them.\n\nFurther actions can be "
 			"found in the context-menu of a computer or "
 			"classroom.\n" ),
@@ -139,9 +151,33 @@ classroomManager::classroomManager( mainWindow * _main_window, QWidget * _parent
 	help_txt->setWordWrap( TRUE );
 	help_txt->setFont( f );
 
+	m_exportToFileBtn = new QPushButton(
+			QPixmap( ":/resources/filesave.png" ),
+			tr( "Export to text-file" ),
+			contentParent() );
+	contentParent()->layout()->addWidget( m_exportToFileBtn );
+	connect( m_exportToFileBtn, SIGNAL( clicked() ),
+			this, SLOT( clickedExportToFile() ) );
+	m_exportToFileBtn->setWhatsThis( tr( "Use this button for exporting "
+				"this list of computers and usernames into "
+				"a text-file. You can use this file "
+				"later for collecting files "
+				"after an exam has finished. "
+				"This is sometimes neccessary, "
+				"because some users might have "
+				"finished and logged out "
+				"earlier and so you cannot "
+				"collect their files at the "
+				"end of the exam." ) );
+
 	loadGlobalClientConfig();
 	loadPersonalConfig();
 
+	QAction * hide_teacher_clients =
+		m_quickSwitchMenu->addAction( tr( "Hide teacher computers" ) );
+	connect( hide_teacher_clients, SIGNAL( triggered( bool ) ),
+					this, SLOT( hideTeacherClients() ) );
+
 	show();
 }
 
@@ -207,7 +243,7 @@ void classroomManager::saveGlobalClientConfig( void )
 					"write-access to your home-directory "
 					"and to %1 (if already existing)."
 					).arg( ITALC_CONFIG_PATH
-						).toAscii().constData() );
+						).toUtf8().constData() );
 	}*/
 
 	QFile( m_globalClientConfiguration + ".bak" ).remove();
@@ -216,7 +252,7 @@ void classroomManager::saveGlobalClientConfig( void )
 	QFile outfile( m_globalClientConfiguration );
 	outfile.open( QFile::WriteOnly | QFile::Truncate );
 
-	outfile.write( xml.toAscii() );
+	outfile.write( xml.toUtf8() );
 	outfile.close();
 }
 
@@ -239,20 +275,28 @@ void classroomManager::savePersonalConfig( void )
 						m_clientUpdateInterval );
 	globalsettings.setAttribute( "win-width", getMainWindow()->width() );
 	globalsettings.setAttribute( "win-height", getMainWindow()->height() );
-	globalsettings.setAttribute( "ismaximized", getMainWindow()->isMaximized() );
+	globalsettings.setAttribute( "win-x", getMainWindow()->x() );
+	globalsettings.setAttribute( "win-y", getMainWindow()->y() );	
+	globalsettings.setAttribute( "ismaximized",
+					getMainWindow()->isMaximized() );
 	globalsettings.setAttribute( "opened-tab",
 				getMainWindow()->m_sideBar->openedTab() );
 
 	globalsettings.setAttribute( "wincfg", QString(
 				getMainWindow()->saveState().toBase64() ) );
 
-	globalsettings.setAttribute( "demo-net-iface", __demo_network_interface );
+	globalsettings.setAttribute( "demo-net-iface",
+						__demo_network_interface );
 	globalsettings.setAttribute( "demo-master-ip", __demo_master_ip );
 	globalsettings.setAttribute( "defaultdomain", __default_domain );
 	globalsettings.setAttribute( "demoquality", __demo_quality );
 	globalsettings.setAttribute( "role", __role );
 	globalsettings.setAttribute( "notooltips",
 					toolButton::toolTipsDisabled() );
+	globalsettings.setAttribute( "clientdoubleclickaction",
+						m_clientDblClickAction );
+	globalsettings.setAttribute( "showUserColumn",
+						m_showUserColumnCheckBox->isChecked() );
 
 	head.appendChild( globalsettings );
 
@@ -270,20 +314,22 @@ void classroomManager::savePersonalConfig( void )
 	QString xml = "<?xml version=\"1.0\"?>\n" + doc.toString( 2 );
 	if( mainWindow::ensureConfigPathExists() == FALSE )
 	{
-		qWarning( QString( "Could not read/write or create directory %1!"
-					"For running iTALC, make sure you have "
-					"write-access to your home-directory "
-					"and to %1 (if already existing)."
+		qWarning( QString( "Could not read/write or create directory "
+					"%1! For running iTALC, make sure you "
+					"have write-access to your home-"
+					"directory and to %1 (if already "
+					"existing)."
 				).arg( localSystem::personalConfigDir()
-						).toAscii().constData() );
+						).toUtf8().constData() );
 	}
 
 	QFile( m_personalConfiguration + ".bak" ).remove();
-	QFile( m_personalConfiguration ).copy( m_personalConfiguration + ".bak" );
+	QFile( m_personalConfiguration ).copy( m_personalConfiguration +
+								".bak" );
 	QFile outfile( m_personalConfiguration );
 	outfile.open( QFile::WriteOnly | QFile::Truncate );
 
-	outfile.write( xml.toAscii() );
+	outfile.write( xml.toUtf8() );
 	outfile.close();
 }
 
@@ -324,19 +370,19 @@ void classroomManager::saveSettingsOfChildren( QDomDocument & _doc,
 								c->name() );
 					client_element.setAttribute( "localip",
 								c->localIP() );
-					client_element.setAttribute( "remoteip",
-								c->remoteIP() );
 					client_element.setAttribute( "mac",
 								c->mac() );
+					client_element.setAttribute( "type",
+								c->type() );
 				}
 				else
 				{
 					client_element.setAttribute( "visible",
 						c->isVisible() ? "yes" : "no" );
 						client_element.setAttribute(
-			"x", QString::number( c->parentWidget()->pos().x() ) );
+			"x", QString::number( c->pos().x() ) );
  						client_element.setAttribute(
-			"y", QString::number( c->parentWidget()->pos().y() ) );
+			"y", QString::number( c->pos().y() ) );
 						client_element.setAttribute(
 			"w", QString::number( c->width() ) );
  						client_element.setAttribute(
@@ -366,6 +412,25 @@ QVector<client *> classroomManager::visibleClients( void ) const
 
 
 
+QVector<client *> classroomManager::getLoggedInClients( void ) const
+{
+	QVector<client *> loggedClients;
+
+	// loop through all clients
+	foreach( client * it, visibleClients() )
+	{
+		const QString user = it->user();
+		if( user != "none" && !user.isEmpty() )
+		{
+			loggedClients.push_back( it );
+		}
+	}
+	return( loggedClients );
+}
+
+
+
+
 void classroomManager::getVisibleClients( QTreeWidgetItem * _p,
 						QVector<client *> & _vc )
 {
@@ -399,22 +464,25 @@ void classroomManager::getHeaderInformation( const QDomElement & _header )
 		{
 			m_clientUpdateInterval = node.toElement().attribute(
 					"client-update-interval" ).toInt();
-/*			if( node.toElement().attribute( "use-big-icons" ) !=
-								QString::null )
-			{
-				italc::inst()->setUsesBigPixmaps(
-					node.toElement().attribute(
-						"use-big-icons" ).toInt() );
-			}*/
 			if( node.toElement().attribute( "win-width" ) !=
 								QString::null &&
 				node.toElement().attribute( "win-height" ) !=
+								QString::null &&
+				node.toElement().attribute( "win-x" ) !=
+								QString::null &&
+				node.toElement().attribute( "win-y" ) !=
 								QString::null )
 			{
-				getMainWindow()->resize(
-			node.toElement().attribute( "win-width" ).toInt(),
-			node.toElement().attribute( "win-height" ).toInt() );
+getMainWindow()->resize( node.toElement().attribute( "win-width" ).toInt(),
+			node.toElement().attribute("win-height" ).toInt() );
+
+getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
+				node.toElement().attribute( "win-y" ).toInt() );
 			}
+			else
+			{
+				setDefaultWindowsSizeAndPosition();
+ 			}
 			if( node.toElement().attribute( "opened-tab" ) !=
 								QString::null )
 			{
@@ -422,10 +490,11 @@ void classroomManager::getHeaderInformation( const QDomElement & _header )
 						node.toElement().attribute(
 							"opened-tab" ).toInt();
 			}
-			if( node.toElement().attribute( "ismaximized" ).toInt() > 0 )
+			if( node.toElement().attribute( "ismaximized" ).
+								toInt() > 0 )
 			{
-				getMainWindow()->setWindowState( getMainWindow()->windowState() |
-										Qt::WindowMaximized );
+	getMainWindow()->setWindowState( getMainWindow()->windowState() |
+							Qt::WindowMaximized );
 			}
 			if( node.toElement().attribute( "wincfg" ) !=
 								QString::null )
@@ -457,11 +526,15 @@ void classroomManager::getHeaderInformation( const QDomElement & _header )
 			toolButton::setToolTipsDisabled(
 				node.toElement().attribute( "notooltips" ).
 								toInt() );
+			m_clientDblClickAction = node.toElement().attribute(
+					"clientdoubleclickaction" ).toInt();
+			m_showUserColumnCheckBox->setChecked(
+				node.toElement().attribute( "showUserColumn" ).toInt() );
 			// if the attr did not exist, we got zero as value,
 			// which is not acceptable
 			if( m_clientUpdateInterval < 1 )
 			{
-				m_clientUpdateInterval = 1;
+				m_clientUpdateInterval = 2;
 			}
 		}
 		node = node.nextSibling();
@@ -518,25 +591,16 @@ void classroomManager::loadTree( classRoom * _parent_item,
 				// add new client
 				QString local_ip = node.toElement().attribute(
 								"localip" );
-
-				// compat-code - to be removed with final 1.0.0
-				if( local_ip == QString::null )
-				{
-					local_ip = node.toElement().attribute(
-									"ip" );
-				}
-				QString remote_ip = node.toElement().attribute(
-								"remoteip" );
 				QString mac = node.toElement().attribute(
 									"mac" );
 				QString name = node.toElement().attribute(
 								"name" );
 
-				c = new client( local_ip, remote_ip, mac,
-							name, _parent_item,
-							getMainWindow(),
-						node.toElement().attribute(
-							"id" ).toInt() );
+	c = new client( local_ip, mac, name,
+				(client::types) node.toElement().
+						attribute( "type" ).toInt(),
+			_parent_item, getMainWindow(), node.toElement().
+						attribute( "id" ).toInt() );
 				c->hide();
 			}
 			else
@@ -548,7 +612,7 @@ void classroomManager::loadTree( classRoom * _parent_item,
 				{
 					continue;
 				}
-				c->parentWidget()->move(
+				c->move(
 			node.toElement().attribute( "x" ).toInt(),
 			node.toElement().attribute( "y" ).toInt() );
 				c->m_rasterX = node.toElement().attribute(
@@ -650,6 +714,15 @@ void classroomManager::loadGlobalClientConfig( void )
 
 
 
+void classroomManager::setDefaultWindowsSizeAndPosition( void )
+{
+	getMainWindow()->resize( DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT );
+	getMainWindow()->move( QPoint( 0, 0 ) );	
+}
+
+
+
+
 void classroomManager::loadPersonalConfig( void )
 {
 	if( !QFileInfo( m_personalConfiguration ).exists() &&
@@ -663,6 +736,7 @@ void classroomManager::loadPersonalConfig( void )
 	QFile cfg_file( m_personalConfiguration );
 	if( !cfg_file.open( QIODevice::ReadOnly ) )
 	{
+		setDefaultWindowsSizeAndPosition();
 		return;
 	}
 
@@ -680,6 +754,7 @@ void classroomManager::loadPersonalConfig( void )
 					).arg( m_personalConfiguration ),
 					QPixmap( ":/resources/error.png" ) );
 		cfg_file.close();
+		setDefaultWindowsSizeAndPosition();
 		return;
 	}
 	cfg_file.close();
@@ -774,6 +849,34 @@ void classroomManager::updateClients( void )
 
 
 
+void classroomManager::clickedExportToFile( void )
+{
+	QString outfn = QFileDialog::getSaveFileName( this,
+			tr( "Select output-file" ),
+			QDir::homePath(),
+			tr( "Text files (*.txt)" ) );
+	if( outfn == "" )
+	{
+		return;
+	}
+
+	QString output = "# " + QDateTime::currentDateTime().toString() + "\n";
+
+	QVector<client *> clients = getLoggedInClients();
+	foreach( client * cl, clients) 
+	{
+		output += cl->user() + "\t@ " + cl->name() + " [" + cl->localIP() + "]\n";
+	}
+
+	QFile outfile( outfn );
+	outfile.open( QFile::WriteOnly );
+	outfile.write( output.toUtf8() );
+	outfile.close();
+}
+
+
+
+
 inline void classroomManager::cmdToVisibleClients( client::clientCmds _cmd,
 						const QString & _u_data )
 {
@@ -812,19 +915,19 @@ void classroomManager::powerOnClients( void )
 {
 /*	progressInformation pi(
 		tr( "Please wait, while the clients are being turned on." ) );*/
-	cmdToVisibleClients( client::PowerOn );
+	cmdToVisibleClients( client::Cmd_PowerOn );
 }
 
 
 
 
-void classroomManager::multiLogon( void )
+void classroomManager::remoteLogon( void )
 {
-	multiLogonDialog mld( this );
+	remoteLogonDialog mld( this );
 	if( mld.exec() == QDialog::Accepted &&
 		!mld.userName().isEmpty() && !mld.password().isEmpty() )
 	{
-		cmdToVisibleClients( client::LogonUserCmd,
+		cmdToVisibleClients( client::Cmd_LogonUser,
 				mld.userName() + "*" + mld.password() +
 							"*" + mld.domain() );
 	}
@@ -843,7 +946,7 @@ void classroomManager::powerDownClients( void )
 	{
 		/*progressInformation pi( tr( "Please wait, while the computers "
 						"are being powered down." ) );*/
-		cmdToVisibleClients( client::PowerDown );
+		cmdToVisibleClients( client::Cmd_PowerDown );
 	}
 }
 
@@ -858,7 +961,7 @@ void classroomManager::logoutUser( void )
 					QMessageBox::Yes, QMessageBox::No ) ==
 							QMessageBox::Yes )
 	{
-		cmdToVisibleClients( client::LogoutUser );
+		cmdToVisibleClients( client::Cmd_LogoutUser );
 	}
 }
 
@@ -872,7 +975,7 @@ void classroomManager::execCmds( void )
 	cmdInputDialog cmd_input_dialog( cmds, this );
 	if( cmd_input_dialog.exec() == QDialog::Accepted && cmds != "" )
 	{
-		cmdToVisibleClients( client::ExecCmds, cmds );
+		cmdToVisibleClients( client::Cmd_ExecCmds, cmds );
 	}
 }
 
@@ -888,108 +991,15 @@ void classroomManager::sendMessage( void )
 	{
 		/*progressInformation pi(
 			tr( "Please wait, while the message is being sent." ) );*/
-		cmdToVisibleClients( client::SendTextMessage, msg );
+		cmdToVisibleClients( client::Cmd_SendTextMessage, msg );
 	}
 }
 
 
 
-void classroomManager::distributeFile( void )
-{
-/*	QMessageBox::information (this, tr("Function not implemented yet."), tr("This function is not completely implemented yet. This is why it is disabled at the moment."), QMessageBox::Ok);
-	return;*/
-#if 0
-	QFileDialog ofd( this, client::tr( "Select file to distribute" ),
-							QDir::homePath() );
-	ofd.setFileMode( QFileDialog::ExistingFile );
-	if( ofd.exec() == QDialog::Accepted &&
-					ofd.selectedFiles().empty() == FALSE )
-	{
-		cmdToVisibleClients( client::DistributeFile,
-						ofd.selectedFiles().front() );
-	}
-#endif
-}
 
-
-
-
-void classroomManager::collectFiles( void )
-{
-/*	QMessageBox::information (this, tr("Function not implemented yet."), tr("This function is not completely implemented yet. This is why it is disabled at the moment."), QMessageBox::Ok);
-	return;*/
-
-#if 0
-	// ask user for file(s)
-	bool ok;
-	QString f = QInputDialog::getText( this, tr( "Collect files" ),
-					client::tr( "Please enter the name(s) "
-							"of the file(s) to be "
-							"collected (wildcards "
-							"are allowed).\nThe "
-							"base-directory is "
-							"HOME/PUBLIC." ),
-						QLineEdit::Normal, "", &ok );
-	if( ok && !f.isEmpty() )
-	{
-		cmdToVisibleClients( client::CollectFiles, f );
-	}
-#endif
-}
-
-
-
-
-void classroomManager::collectFilesFromUserList( void )
-{
-	QMessageBox::information (this, tr("Function not implemented yet."), tr("This function is not completely implemented yet. This is why it is disabled at the moment."), QMessageBox::Ok);
-	return;
-/*	// ask user for user-list
-	QString infile = QFileDialog::getOpenFileName( this,
-					tr( "Select exported user-list" ),
-					QDir::homePath() );
-	if( infile == QString::null )
-	{
-		return;
-	}
-
-	// ask user for file(s)
-	QString fn;
-	textInputDialog file_name_input( client::tr( "Please enter the name(s) "
-							"of the file(s) to be "
-							"collected (wildcards "
-							"are allowed).\nThe "
-							"base-directory is "
-							"HOME/PUBLIC." ),
-								fn, this );
-	file_name_input.setWindowTitle( client::tr( "Collect files" ) );
-
-	if( file_name_input.exec() == QDialog::Accepted && fn != "" )
-	{
-		// open and read user-list
-		QFile f( infile );
-		f.open( QFile::ReadOnly );
-		QTextStream ts( &f );
-		ts.readLine();			// skip date which is located
-						// in first line of exported
-						// user-list
-		while( ts.atEnd() == FALSE )
-		{
-			// extract user-name in brackets
-			QString user = ts.readLine().section( '(', -1, -1
-							).section( ')', 0, 1 );
-#warning TODO: make local ISD run with priveleges to read out of users home-dir?
-			getMainWindow()->localISD()->
-			// collect files from this user
-			systemEnvironment::collectFiles( fn, user );
-		}
-	}*/
-}
-
-
-
-const int decor_w = 8;
-const int decor_h = 34;
+const int decor_w = 2;
+const int decor_h = 2;
 
 
 void classroomManager::adjustWindows( void )
@@ -997,8 +1007,10 @@ void classroomManager::adjustWindows( void )
 	QVector<client *> vc = visibleClients();
 	if( vc.size() )
 	{
-		const int avail_w = getMainWindow()->workspace()->width();
-		const int avail_h = getMainWindow()->workspace()->height();
+		const int avail_w = getMainWindow()->workspace()->
+						parentWidget()->width();
+		const int avail_h = getMainWindow()->workspace()->
+						parentWidget()->height();
 		float cw = vc[0]->width() + decor_w;// add width of decoration
 		float ch = vc[0]->height() + decor_h;// add height of titlebar
 
@@ -1012,18 +1024,18 @@ void classroomManager::adjustWindows( void )
 		{
 			ch = 192.0f;
 		}
-		int x_offset = vc[0]->parentWidget()->pos().x();
-		int y_offset = vc[0]->parentWidget()->pos().y();
+		int x_offset = vc[0]->pos().x();
+		int y_offset = vc[0]->pos().y();
 
 		foreach( client * cl, vc )
 		{
-			if( cl->parentWidget()->pos().x() < x_offset )
+			if( cl->pos().x() < x_offset )
 			{
-				x_offset = cl->parentWidget()->pos().x();
+				x_offset = cl->pos().x();
 			}
-			if( cl->parentWidget()->pos().y() < y_offset )
+			if( cl->pos().y() < y_offset )
 			{
-				y_offset = cl->parentWidget()->pos().y();
+				y_offset = cl->pos().y();
 			}
 		}
 
@@ -1032,9 +1044,9 @@ void classroomManager::adjustWindows( void )
 		foreach( client * cl, vc )
 		{
 			cl->m_rasterX = roundCorrect( (
-				cl->parentWidget()->pos().x()-x_offset ) / cw );
+					cl->pos().x()-x_offset ) / cw );
 			cl->m_rasterY = roundCorrect( (
-				cl->parentWidget()->pos().y()-y_offset ) / ch );
+					cl->pos().y()-y_offset ) / ch );
 			if( cl->m_rasterX > max_rx )
 			{
 				max_rx = cl->m_rasterX;
@@ -1066,10 +1078,10 @@ void classroomManager::adjustWindows( void )
 		foreach( client * cl, vc )
 		{
 			cl->setFixedSize( nw - decor_w, nh - decor_h );
-			cl->parentWidget()->move(
-				static_cast<int>( cl->m_rasterX * nw ),
+			cl->move( static_cast<int>( cl->m_rasterX * nw ),
 				static_cast<int>( cl->m_rasterY * nh ) );
 		}
+		getMainWindow()->workspace()->updateGeometry();
 	}
 }
 
@@ -1078,7 +1090,7 @@ void classroomManager::adjustWindows( void )
 
 bool clientIPSort( const client * _1, const client * _2 )
 {
-	return( _1->remoteIP() < _2->remoteIP() );
+	return( _1->localIP() < _2->localIP() );
 }
 
 
@@ -1089,13 +1101,15 @@ void classroomManager::arrangeWindows( void )
 	QVector<client *> vc = visibleClients();
 	if( vc.size() )
 	{
-		const int avail_w = getMainWindow()->workspace()->width();
-		const int avail_h = getMainWindow()->workspace()->height();
+		const int avail_w = getMainWindow()->workspace()->
+						parentWidget()->width();
+		const int avail_h = getMainWindow()->workspace()->
+						parentWidget()->height();
 		const int w = avail_w;
 		const int h = avail_h;
-		const float s = sqrt( vc.size() * h / (float) w );
-		const int win_per_line = (int) round( vc.size() / round( s ) );
-		const int win_per_row = (int) round( s );
+		const float s = sqrt( vc.size() *3* w / (float) (4*h) );
+		int win_per_row = (int) ceil( vc.size() / s );
+		int win_per_line = (int) ceil( s );
 		const int ww = avail_w / win_per_line;
 		const int wh = avail_h / win_per_row;
 
@@ -1104,8 +1118,7 @@ void classroomManager::arrangeWindows( void )
 		int i = 0;
 		foreach( client * cl, vc )
 		{
-			cl->parentWidget()->move( ( i % win_per_line ) *
-							( ww + decor_w ),
+			cl->move( ( i % win_per_line ) * ( ww + decor_w ),
 						( i / win_per_line ) *
 							( wh + decor_h ) );
 			cl->setFixedSize( ww, wh );
@@ -1140,18 +1153,18 @@ void classroomManager::resizeClients( const int _new_width )
 			ch = 192.0f;
 		}
 
-		int x_offset = vc[0]->parentWidget()->pos().x();
-		int y_offset = vc[0]->parentWidget()->pos().y();
+		int x_offset = vc[0]->pos().x();
+		int y_offset = vc[0]->pos().y();
 
 		foreach( client * cl, vc )
 		{
-			if( cl->parentWidget()->pos().x() < x_offset )
+			if( cl->pos().x() < x_offset )
 			{
-				x_offset = cl->parentWidget()->pos().x();
+				x_offset = cl->pos().x();
 			}
-			if( cl->parentWidget()->pos().y() < y_offset )
+			if( cl->pos().y() < y_offset )
 			{
-				y_offset = cl->parentWidget()->pos().y();
+				y_offset = cl->pos().y();
 			}
 		}
 
@@ -1159,12 +1172,12 @@ void classroomManager::resizeClients( const int _new_width )
 		{
 			cl->setFixedSize( _new_width, _new_height );
 			const int xp = static_cast<int>( (
-				cl->parentWidget()->pos().x() - x_offset ) /
+				cl->pos().x() - x_offset ) /
 				cw * ( _new_width + decor_w ) ) + x_offset;
 			const int yp = static_cast<int>( (
-				cl->parentWidget()->pos().y() - y_offset ) /
+				cl->pos().y() - y_offset ) /
 				ch * ( _new_height + decor_h ) ) + y_offset;
-			cl->parentWidget()->move( xp, yp );
+			cl->move( xp, yp );
 		}
 	}
 }
@@ -1267,13 +1280,21 @@ void classroomManager::contextMenuRequest( const QPoint & _pos )
 
 	if( cri != NULL )
 	{
+		QMenu * single_clients_submenu = context_menu->addMenu(
+					QPixmap( ":/resources/client.png" ),
+							tr( "Actions" ) );
+
+		cri->getClient()->createActionMenu( single_clients_submenu );
+
+		context_menu->addSeparator();
+
 		context_menu->addAction(
 				QPixmap( ":/resources/client_show.png" ),
 						tr( "Show/hide" ),
 					this, SLOT( showHideClient() ) );
 
 		context_menu->addAction(
-				QPixmap( ":/resources/client_settings.png" ),
+				QPixmap( ":/resources/config.png" ),
 						tr( "Edit settings" ), this,
 					SLOT( editClientSettings() ) );
 
@@ -1284,16 +1305,16 @@ void classroomManager::contextMenuRequest( const QPoint & _pos )
 
 		context_menu->addSeparator();
 
-		QMenu * single_clients_submenu = context_menu->addMenu(
-					QPixmap( ":/resources/client.png" ),
-							tr( "Actions" ) );
-
-		cri->getClient()->createActionMenu( single_clients_submenu );
-
-		context_menu->addSeparator();
 	}
 	else if( cr != NULL )
 	{
+		QMenu * actions_for_classroom_submenu = context_menu->addMenu(
+				QPixmap( ":/resources/classroom_opened.png" ),
+				tr( "Actions for %1" ).arg( cr->text( 0 ) ) );
+		cr->createActionMenu( actions_for_classroom_submenu, FALSE );
+
+		context_menu->addSeparator();
+
 		context_menu->addAction(
 				QPixmap( ":/resources/classroom_show.png" ),
 				tr( "Show all computers in classroom" ),
@@ -1305,7 +1326,12 @@ void classroomManager::contextMenuRequest( const QPoint & _pos )
 				this, SLOT( hideSelectedClassRooms() ) );
 
 		context_menu->addAction(
-				QPixmap( ":/resources/client_settings.png" ),
+				QPixmap( ":/resources/no_user.png" ),
+				tr( "Hide teacher computers" ),
+				this, SLOT( hideTeacherClients() ) );
+
+		context_menu->addAction(
+				QPixmap( ":/resources/config.png" ),
 				tr( "Edit name" ),
 				this, SLOT( editClassRoomName() ) );
 
@@ -1316,13 +1342,6 @@ void classroomManager::contextMenuRequest( const QPoint & _pos )
 
 		context_menu->addSeparator();
 
-		QMenu * actions_for_classroom_submenu = context_menu->addMenu(
-				QPixmap( ":/resources/classroom_opened.png" ),
-				tr( "Actions for %1" ).arg( cr->text( 0 ) ) );
-		cr->createActionMenu( actions_for_classroom_submenu, FALSE );
-
-		context_menu->addSeparator();
-
 	}
 	else if( m_classRooms.size() )
 	{ 
@@ -1450,8 +1469,8 @@ void classroomManager::removeClient( void )
 		foreach( classRoomItem * cri, si )
 		{
 			cri->getClient()->hide();
-			m_clientsToRemove.push_back( cri->getClient() );
 			m_view->setItemHidden( cri, TRUE );
+			m_clientsToRemove.push_back( cri->getClient() );
 		}
 	}
 }
@@ -1461,6 +1480,15 @@ void classroomManager::removeClient( void )
 
 void classroomManager::setStateOfClassRoom( classRoom * _cr, bool _shown )
 {
+	if( _shown )
+	{
+		_cr->setMenuItemIcon( QIcon( ":/resources/greenled.png" ) );
+	}
+	else
+	{
+		_cr->setMenuItemIcon( QIcon() );
+	}
+
 	// If all clients are shown, we hide them all. Otherwise we show all.
 	for( int i = 0; i < _cr->childCount(); ++i )
 	{
@@ -1478,8 +1506,11 @@ void classroomManager::setStateOfClassRoom( classRoom * _cr, bool _shown )
 
 QAction * classroomManager::addClassRoomToQuickSwitchMenu( classRoom * _cr )
 {
-	return( m_quickSwitchMenu->addAction( _cr->text( 0 ), _cr,
-						SLOT( switchToClassRoom() ) ) );
+	QAction * a = new QAction( _cr->text( 0 ), m_quickSwitchMenu );
+	connect( a, SIGNAL( triggered( bool ) ), _cr,
+						SLOT( switchToClassRoom() ) );
+	m_quickSwitchMenu->insertAction( m_qsmClassRoomSeparator, a );
+	return( a );
 }
 
 
@@ -1670,27 +1701,51 @@ void classroomManager::addClassRoom( void )
 
 
 
+void classroomManager::hideTeacherClients( void )
+{
+	QVector<client *> vc = visibleClients();
+
+	foreach( client * cl, vc )
+	{
+		if( cl->type() == client::Type_Teacher )
+		{
+			cl->hide();
+		}
+	}
+}
 
 
 
 
+void classroomManager::showUserColumn( int _show )
+{
+	m_view->showColumn( _show ? 2 : 1 );
+	m_view->hideColumn( _show ? 1 : 2 );
+}
 
-classRoom::classRoom( const QString & _name, classroomManager * _classroom_manager,
+
+
+
+classRoom::classRoom( const QString & _name,
+					classroomManager * _classroom_manager,
 						QTreeWidgetItem * _parent ) :
 	QTreeWidgetItem( _parent, QStringList( _name ) ),
 	m_classroomManager( _classroom_manager ),
-	m_qsMenuAction( m_classroomManager->addClassRoomToQuickSwitchMenu( this ) )
+	m_qsMenuAction( m_classroomManager->addClassRoomToQuickSwitchMenu(
+									this ) )
 {
 }
 
 
 
 
-classRoom::classRoom( const QString & _name, classroomManager * _classroom_manager,
+classRoom::classRoom( const QString & _name,
+					classroomManager * _classroom_manager,
 						QTreeWidget * _parent ) :
 	QTreeWidgetItem( _parent, QStringList( _name ) ),
 	m_classroomManager( _classroom_manager ),
-	m_qsMenuAction( m_classroomManager->addClassRoomToQuickSwitchMenu( this ) )
+	m_qsMenuAction( m_classroomManager->addClassRoomToQuickSwitchMenu(
+									this ) )
 {
 }
 
@@ -1704,26 +1759,6 @@ classRoom::~classRoom()
 
 
 
-/*
-void classRoom::initPixmaps( void )
-{
-	if( s_classRoomClosedPixmap == NULL )
-	{
-		s_classRoomClosedPixmap = new QPixmap(
-					":/resources/classroom_closed.png" );
-	}
-
-	if( s_classRoomOpenedPixmap == NULL )
-	{
-		s_classRoomOpenedPixmap = new QPixmap(
-					":/resources/classroom_opened.png" );
-	}
-
-	setIcon( *s_classRoomClosedPixmap, 0 );
-}*/
-
-
-
 
 void classRoom::createActionMenu( QMenu * _m, bool _add_sub_menu )
 {
@@ -1737,14 +1772,15 @@ void classRoom::createActionMenu( QMenu * _m, bool _add_sub_menu )
 	}
 	connect( this_classroom_submenu, SIGNAL( triggered( QAction * ) ),
 			this, SLOT( processCmdOnAllClients( QAction * ) ) );
-	for( int i = client::SendTextMessage; i < client::CmdCount; ++i )
+	for( int i = client::Cmd_SendTextMessage; i < client::Cmd_CmdCount;
+									++i )
 	{
-		if( i == client::ExecCmds )
+		if( i == client::Cmd_ExecCmds )
 		{
 			this_classroom_submenu =
 				this_classroom_submenu->addMenu( QPixmap(
-					":/resources/client_settings.png" ),
-							tr( "Administation" ) );
+					":/resources/config.png" ),
+						tr( "Administration" ) );
 
 			connect( this_classroom_submenu,
 					SIGNAL( triggered( QAction * ) ),
@@ -1781,7 +1817,7 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 	// may be we have to do something (e.g. confirm) before executing cmd
 	switch( static_cast<client::clientCmds>( cmd ) )
 	{
-		case client::SendTextMessage:
+		case client::Cmd_SendTextMessage:
 		{
 			textMessageDialog tmd( u_data, m_classroomManager );
 			if( tmd.exec() != QDialog::Accepted || u_data == "" )
@@ -1791,7 +1827,21 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 			break;
 		}
 
-		case client::LogoutUser:
+		case client::Cmd_LogonUser:
+		{
+			remoteLogonDialog mld( m_classroomManager );
+			if( mld.exec() == QDialog::Accepted &&
+				!mld.userName().isEmpty() &&
+					!mld.password().isEmpty() )
+			{
+				u_data = mld.userName() + "*" + mld.password() +
+							"*" + mld.domain();
+				break;
+			}
+			return;
+		}
+
+		case client::Cmd_LogoutUser:
 		{
 			if( QMessageBox::question( m_classroomManager->window(),
 					classroomManager::tr( "Logout user" ),
@@ -1807,7 +1857,7 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 			break;
 		}
 
-		case client::ExecCmds:
+		case client::Cmd_ExecCmds:
 		{
 			cmdInputDialog cmd_input_dialog( u_data, NULL );
 			if( cmd_input_dialog.exec() != QDialog::Accepted ||
@@ -1818,22 +1868,7 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 			break;
 		}
 
-/*		case client::SSHLogin:
-		{
-			bool ok;
-			u_data = QInputDialog::getText( NULL,
-				client::tr( "SSH-login" ),
-				client::tr( "Please enter the user-name you "
-							"want to login with." ),
-					QLineEdit::Normal, "root", &ok );
-			if( !ok || u_data.isEmpty() )
-			{
-				return;
-			}
-			break;
-		}*/
-
-		case client::Reboot:
+		case client::Cmd_Reboot:
 		{
 			if( QMessageBox::question( m_classroomManager->window(),
 				classroomManager::tr( "Reboot computers" ),
@@ -1846,12 +1881,13 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 			}
 			break;
 		}
-		case client::PowerDown:
+		case client::Cmd_PowerDown:
 		{
 			if( QMessageBox::question( m_classroomManager->window(),
 				classroomManager::tr( "Power down computers" ),
-				classroomManager::tr( "Are you sure want to power "
-						"down all visible computers?" ),
+				classroomManager::tr( "Are you sure want to "
+						"power down all visible "
+								"computers?" ),
 					QMessageBox::Yes, QMessageBox::No ) ==
 							QMessageBox::No )
 			{
@@ -1872,30 +1908,6 @@ void classRoom::processCmdOnAllClients( QAction * _action )
 
 
 
-/*
-void classRoom::setPixmap( QPixmap * _pix )
-{
-	m_pix = _pix;
-}
-
-
-
-
-void classRoom::setExpanded( bool _o )
-{
-	if( _o )
-	{
-		setPixmap( s_classRoomOpenedPixmap );
-	}
-	else
-	{
-		setPixmap( s_classRoomClosedPixmap );
-	}
-
-	treeWidget()->setItemExpanded( this, _o );
-}
-*/
-
 
 
 void classRoom::switchToClassRoom( void )
@@ -1928,8 +1940,9 @@ classRoomItem::classRoomItem( client * _client, QTreeWidgetItem * _parent ) :
 					":/resources/client_visible.png" );
 	}
 
-	setVisible( FALSE );
-	setText( 1, m_client->remoteIP() );
+	setVisible( m_client->isVisible() );
+	setText( 1, m_client->localIP() );
+	setUser( m_client->user() );
 }
 
 
@@ -1958,5 +1971,13 @@ void classRoomItem::setVisible( const bool _obs )
 
 
 
+void classRoomItem::setUser( const QString & _name )
+{
+	setText( 2, _name );
+}
+
+
+
+
 #include "classroom_manager.moc"
 
