@@ -2,7 +2,7 @@
  * client.cpp - code for client-windows, which are displayed in several
  *              instances in the main-window of iTALC
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -24,22 +24,12 @@
  */
 
  
-#include <QtCore/QDateTime>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtGui/QApplication>
 #include <QtGui/QCloseEvent>
-#include <QtGui/QFileDialog>
-#include <QtGui/QImage>
-#include <QtGui/QInputDialog>
 #include <QtGui/QLinearGradient>
 #include <QtGui/QMenu>
-#include <QtGui/QMessageBox>
 #include <QtGui/QPainter>
 #include <QtGui/QPixmap>
-#include <QtGui/QToolBar>
-#include <QtGui/QToolButton>
-#include <QtGui/QWorkspace>
+#include <QtGui/QScrollArea>
 
 
 #include "main_window.h"
@@ -51,30 +41,33 @@
 #include "snapshot_list.h"
 #include "dialogs.h"
 #include "messagebox.h"
-#include "qnetworkinterface.h"
+#include "remote_control_widget.h"
 
 
 
 const QSize DEFAULT_CLIENT_SIZE( 256, 192 );
+const int DECO_WIDTH = 4;
+const int TITLE_HEIGHT = 20;
+const QPoint CONTENT_OFFSET( DECO_WIDTH, DECO_WIDTH + TITLE_HEIGHT ); 
+const QSize CONTENT_SIZE_SUB( 2*DECO_WIDTH, 2*DECO_WIDTH + TITLE_HEIGHT ); 
 
 
-
-const client::clientCommand client::s_commands[client::CmdCount] =
+const client::clientCommand client::s_commands[client::Cmd_CmdCount] =
 {
 
-	{ None,			NULL,				"",								"",			FALSE	},
-	{ Reload,		&client::reload,		"",								"", 			TRUE	},
-	{ ViewLive,		&client::viewLive,		QT_TRANSLATE_NOOP( "client", "View live and fullscreen" ),	"viewmag.png",		FALSE	},
-	{ RemoteControl,	&client::remoteControl,		QT_TRANSLATE_NOOP( "client", "Remote control" ),		"remote_control.png",	FALSE	},
-	{ ClientDemo,		&client::clientDemo,		QT_TRANSLATE_NOOP( "client", "Let student show demo" ),		"client_demo.png",	FALSE	},
-	{ SendTextMessage,	&client::sendTextMessage,	QT_TRANSLATE_NOOP( "client", "Send text-message" ),		"text_message.png",	TRUE	},
-	{ LogonUserCmd,		&client::logonUser,		QT_TRANSLATE_NOOP( "client", "Logon user" ),			"multilogon.png",		FALSE	},
-	{ LogoutUser,		&client::logoutUser,		QT_TRANSLATE_NOOP( "client", "Logout user" ),			"logout.png",		TRUE	},
-	{ Snapshot,		&client::snapshot,		QT_TRANSLATE_NOOP( "client", "Make a snapshot" ),		"snapshot.png", 	TRUE	},
-	{ PowerOn,		&client::powerOn,		QT_TRANSLATE_NOOP( "client", "Power on" ),			"power_on.png",		FALSE	},
-	{ Reboot,		&client::reboot,		QT_TRANSLATE_NOOP( "client", "Reboot" ),			"reboot.png",		FALSE	},
-	{ PowerDown,		&client::powerDown,		QT_TRANSLATE_NOOP( "client", "Power down" ),			"power_off.png",	FALSE	},
-	{ ExecCmds,		&client::execCmds,		QT_TRANSLATE_NOOP( "client", "Execute commands" ),		"run.png", 		FALSE	}
+	{ Cmd_None,		NULL,				(char *) "",								(char *) "",			FALSE	},
+	{ Cmd_Reload,		&client::reload,		(char *) "",								(char *) "", 			TRUE	},
+	{ Cmd_ViewLive,		&client::viewLive,		(char *) QT_TRANSLATE_NOOP( "client", "View live" ),			(char *) "viewmag.png",		FALSE	},
+	{ Cmd_RemoteControl,	&client::remoteControl,		(char *) QT_TRANSLATE_NOOP( "client", "Remote control" ),		(char *) "remote_control.png",	FALSE	},
+	{ Cmd_ClientDemo,	&client::clientDemo,		(char *) QT_TRANSLATE_NOOP( "client", "Let student show demo" ),		(char *) "client_demo.png",	FALSE	},
+	{ Cmd_SendTextMessage,	&client::sendTextMessage,	(char *) QT_TRANSLATE_NOOP( "client", "Send text message" ),		(char *) "text_message.png",	TRUE	},
+	{ Cmd_LogonUser,	&client::logonUser,		(char *) QT_TRANSLATE_NOOP( "client", "Logon user" ),			(char *) "remotelogon.png",	FALSE	},
+	{ Cmd_LogoutUser,	&client::logoutUser,		(char *) QT_TRANSLATE_NOOP( "client", "Logout user" ),			(char *) "logout.png",		TRUE	},
+	{ Cmd_Snapshot,		&client::snapshot,		(char *) QT_TRANSLATE_NOOP( "client", "Take a snapshot" ),		(char *) "snapshot.png", 	TRUE	},
+	{ Cmd_PowerOn,		&client::powerOn,		(char *) QT_TRANSLATE_NOOP( "client", "Power on" ),			(char *) "power_on.png",		FALSE	},
+	{ Cmd_Reboot,		&client::reboot,		(char *) QT_TRANSLATE_NOOP( "client", "Reboot" ),			(char *) "reboot.png",		FALSE	},
+	{ Cmd_PowerDown,	&client::powerDown,		(char *) QT_TRANSLATE_NOOP( "client", "Power down" ),			(char *) "power_off.png",	FALSE	},
+	{ Cmd_ExecCmds,		&client::execCmds,		(char *) QT_TRANSLATE_NOOP( "client", "Execute commands" ),		(char *) "run.png", 		FALSE	}
 
 } ;
 
@@ -85,27 +78,91 @@ QHash<int, client *> client::s_clientIDs;
 bool client::s_reloadSnapshotList = FALSE;
 
 
+class closeButton : public QWidget
+{
+public:
+	closeButton( QWidget * _parent ) :
+		QWidget( _parent ),
+		m_mouseOver( FALSE )
+	{
+		setFixedSize( 18, 18 );
+	}
+	virtual ~closeButton()
+	{
+	}
 
-client::client( const QString & _local_ip, const QString & _remote_ip,
+	virtual void enterEvent( QEvent * _fe )
+	{
+		m_mouseOver = TRUE;
+		QWidget::enterEvent( _fe );
+		update();
+	}
+	virtual void leaveEvent( QEvent * _fe )
+	{
+		m_mouseOver = FALSE;
+		QWidget::leaveEvent( _fe );
+		update();
+	}
+	virtual void paintEvent( QPaintEvent * _pe )
+	{
+		QPainter p( this );
+		p.setRenderHint( QPainter::Antialiasing, TRUE );
+		p.setPen( QColor( 128, 128, 128 ) );
+		p.setBrush( m_mouseOver ? Qt::gray : Qt::white );
+		p.drawRoundRect( QRectF( 0.5, 0.5, 16, 16 ), 20, 20 );
+		QPen pen( QColor( 64, 64, 64 ) );
+
+		pen.setWidth( 3 );
+		p.setPen( pen );
+		p.drawLine( QLineF( 4.5, 4.5,  12.5, 12.5 ) );
+		p.drawLine( QLineF( 4.5, 12.5, 12.5, 4.5 ) );
+	}
+
+	virtual void mousePressEvent( QMouseEvent * )
+	{
+	}
+
+	virtual void mouseReleaseEvent( QMouseEvent * _me )
+	{
+		if( rect().contains( _me->pos() ) )
+		{
+			parentWidget()->close();
+		}
+	}
+
+private:
+	bool m_mouseOver;
+
+} ;
+
+
+
+
+
+client::client( const QString & _local_ip,
 		const QString & _mac, const QString & _name,
-		classRoom * _class_room, mainWindow * _main_window, int _id ) :
+		types _type, classRoom * _class_room,
+					mainWindow * _main_window, int _id ) :
 	QWidget( _main_window->workspace() ),
 	m_mainWindow( _main_window ),
 	m_connection( NULL ),
+	m_clickPoint( -1, -1 ),
+	m_origPos( -1, -1 ),
 	m_name( _name ),
 	m_localIP( _local_ip ),
-	m_remoteIP( _remote_ip ),
 	m_mac( _mac ),
+	m_type( _type ),
 	m_reloadsAfterReset( 0 ),
 	m_mode( Mode_Overview ),
 	m_user( "" ),
 	m_makeSnapshot( FALSE ),
 	m_state( State_Unkown ),
-	m_statePixmap(),
 	m_syncMutex(),
-	m_classRoomItem( NULL )
+	m_classRoomItem( NULL ),
+	m_contextMenu( NULL ),
+	m_updateThread( NULL )
 {
-	m_mainWindow->workspace()->addWindow( this );
+	new closeButton( this );
 
 	if( _id <= 0 || clientFromID( _id ) != NULL )
 	{
@@ -116,15 +173,13 @@ client::client( const QString & _local_ip, const QString & _remote_ip,
 
 	m_classRoomItem = new classRoomItem( this, _class_room );
 
-	// make sure remote-ip is ok and set to local IP if neccessary
-	setRemoteIP( m_remoteIP );
-
 	m_connection = new ivsConnection( m_localIP,
-						ivsConnection::QualityLow);
+						ivsConnection::QualityLow );
 
+	setAttribute( Qt::WA_OpaquePaintEvent );
 	setWindowIcon( QPixmap( ":/resources/classroom_manager.png" ) );
 
-	setWhatsThis( tr( "This is a client-window. It either displays the "
+/*	setWhatsThis( tr( "This is a client-window. It either displays the "
 				"screen of the according client or a message "
 				"about the state of this client (no user "
 				"logged in/powered off) is shown. You can "
@@ -137,36 +192,29 @@ client::client( const QString & _local_ip, const QString & _remote_ip,
 				"(and all other visible) client-windows by "
 				"using the functions for increasing, "
 				"decreasing or optimizing the client-window-"
-								"size." ) );
+								"size." ) );*/
 
 	setFixedSize( DEFAULT_CLIENT_SIZE );
 	//resize( DEFAULT_CLIENT_SIZE );
-	setAttribute( Qt::WA_NoSystemBackground, TRUE );
-	//setAttribute( Qt::WA_PaintUnclipped, TRUE );
-/*	QToolBar * tb = new QToolBar( this );
-	tb->setAutoFillBackground( TRUE );
-	QPalette pal;
-	pal.setBrush( QPalette::Background, QColor( 0, 0, 0, 160 ) );
-	tb->setPalette( pal );
-	tb->setIconSize( QSize( 16, 16 ) );*/
+
 	QMenu * tb = new QMenu( this );
 	m_contextMenu = tb;
 	connect( tb, SIGNAL( triggered( QAction * ) ), this,
 					SLOT( processCmdSlot( QAction * ) ) );
 	tb->addAction( scaled( ":/resources/overview_mode.png", 16, 16 ),
-			tr( "Watch only (stops demo and unlocks screen)" ) )->
-				setData( CmdCount + Mode_Overview );
+			tr( "Overview" ) )->
+					setData( Cmd_CmdCount + Mode_Overview );
 	tb->addAction( scaled( ":/resources/fullscreen_demo.png", 16, 16 ),
 						tr( "Fullscreen demo" ) )->
-				setData( CmdCount + Mode_FullscreenDemo );
+				setData( Cmd_CmdCount + Mode_FullscreenDemo );
 	tb->addAction( scaled( ":/resources/window_demo.png", 16, 16 ),
 						tr( "Window demo" ) )->
-				setData( CmdCount + Mode_WindowDemo );
+				setData( Cmd_CmdCount + Mode_WindowDemo );
 	tb->addAction( scaled( ":/resources/locked.png", 16, 16 ),
 						tr( "Locked display" ) )->
-				setData( CmdCount + Mode_Locked );
+				setData( Cmd_CmdCount + Mode_Locked );
 	tb->addSeparator();
-	for( int i = ViewLive; i < CmdCount; ++i )
+	for( int i = Cmd_ViewLive; i < Cmd_CmdCount; ++i )
 	{
 		QAction * a = tb->addAction( scaled(
 			QString( ":/resources/" ) + s_commands[i].m_icon,
@@ -187,9 +235,9 @@ client::client( const QString & _local_ip, const QString & _remote_ip,
 
 client::~client()
 {
+	changeMode( client::Mode_Overview, m_mainWindow->localISD() );
 	m_updateThread->quit();
 	m_updateThread->wait();
-	changeMode( client::Mode_Overview, m_mainWindow->localISD() );
 	m_updateThread->update();
 	delete m_updateThread;
 
@@ -261,11 +309,11 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 			case Mode_WindowDemo:
 				_conn->demoServerDenyClient( m_localIP );
 				m_updateThread->enqueueCommand(
-						updateThread::StopDemo );
+						updateThread::Cmd_StopDemo );
 				break;
 			case Mode_Locked:
 				m_updateThread->enqueueCommand(
-						updateThread::UnlockScreen );
+						updateThread::Cmd_UnlockScreen );
 				break;
 		}
 		switch( m_mode = _new_mode )
@@ -277,7 +325,7 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 			case Mode_WindowDemo:
 				_conn->demoServerAllowClient( m_localIP );
 				m_updateThread->enqueueCommand(
-						updateThread::StartDemo,
+						updateThread::Cmd_StartDemo,
 	QList<QVariant>()
 			<< _conn->host() + ":" + QString::number(
 						_conn->demoServerPort() )
@@ -286,7 +334,7 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 				break;
 			case Mode_Locked:
 				m_updateThread->enqueueCommand(
-						updateThread::LockScreen );
+						updateThread::Cmd_LockScreen );
 				break;
 		}
 		//m_syncMutex.unlock();
@@ -300,8 +348,8 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 		{
 			_conn->demoServerDenyClient( m_localIP );
 		}
-		m_updateThread->enqueueCommand( updateThread::StopDemo );
-		m_updateThread->enqueueCommand( updateThread::UnlockScreen );
+		m_updateThread->enqueueCommand( updateThread::Cmd_StopDemo );
+		m_updateThread->enqueueCommand( updateThread::Cmd_UnlockScreen );
 	}
 }
 
@@ -320,7 +368,7 @@ void client::setClassRoom( classRoom * _cr )
 
 void client::resetConnection( void )
 {
-	m_updateThread->enqueueCommand( updateThread::ResetConnection,
+	m_updateThread->enqueueCommand( updateThread::Cmd_ResetConnection,
 								m_localIP );
 }
 
@@ -329,24 +377,7 @@ void client::resetConnection( void )
 
 void client::update( void )
 {
-/*	QString u = m_user;
-	if( u.isEmpty() )
-	{
-		u = "none";
-	}
-	if( u.contains( '(' ) && u.contains( ')' ) )
-	{
-		u = u.section( '(', 1, 1 ).section( ')', 0, 0 );
-	}
-	setWindowTitle( u + "@" + fullName() );*/
-	setWindowTitle( m_name + " (" + m_classRoomItem->parent()->text( 0 ) +
-									")" );
-	states cur_state = currentState();
-	if( cur_state != m_state )
-	{
-		m_state = cur_state;
-		updateStatePixmap();
-	}
+	m_state = currentState();
 	QWidget::update();
 }
 
@@ -365,7 +396,7 @@ void client::createActionMenu( QMenu * _m )
 
 void client::processCmd( clientCmds _cmd, const QString & _u_data )
 {
-	if( _cmd < 0 || _cmd >= CmdCount )
+	if( _cmd < 0 || _cmd >= Cmd_CmdCount )
 	{
 		return;
 	}
@@ -379,13 +410,14 @@ void client::processCmd( clientCmds _cmd, const QString & _u_data )
 void client::processCmdSlot( QAction * _action )
 {
 	int a = _action->data().toInt();
-	if( a >= ViewLive && a < CmdCount )
+	if( a >= Cmd_ViewLive && a < Cmd_CmdCount )
 	{
 		processCmd( static_cast<clientCmds>( a ) );
 	}
-	else if( a >= CmdCount+Mode_Overview && a < CmdCount+Mode_Unknown )
+	else if( a >= Cmd_CmdCount+Mode_Overview &&
+						a < Cmd_CmdCount+Mode_Unknown )
 	{
-		changeMode( static_cast<modes>( a-CmdCount ),
+		changeMode( static_cast<modes>( a - Cmd_CmdCount ),
 						m_mainWindow->localISD() );
 	}
 }
@@ -393,60 +425,6 @@ void client::processCmdSlot( QAction * _action )
 
 
 
-void client::updateStatePixmap( void )
-{
-	m_statePixmap = QPixmap( size() );
-	QPainter p( &m_statePixmap );
-
-	QLinearGradient grad( 0, 0, 0, height() );
-	grad.setColorAt( 0, QColor( 224, 224, 224 ) );
-	grad.setColorAt( 1, QColor( 128, 128, 128 ) );
-	p.fillRect( rect(), grad );//QColor( 255, 255, 255 ) );
-
-	QPixmap pm = QPixmap( ":/resources/error.png" );
-	QString msg = tr( "Unknown state" );
-
-	switch( m_state )
-	{
-		case State_Overview:
-			return;
-		case State_NoUserLoggedIn:
-			pm = QPixmap( ":/resources/no_user.png" );
-			msg = tr( "No user logged in" );
-			break;
-		case State_Unreachable:
-			pm = QPixmap( ":/resources/host_unreachable.png" );
-			msg = tr( "Host unreachable" );
-			break;
-		case State_Demo:
-			pm = QPixmap( ":/resources/window_demo.png" );
-			msg = tr( "Demo running" );
-			break;
-		case State_Locked:
-			pm = QPixmap( ":/resources/locked.png" );
-			msg = tr( "Desktop locked" );
-			break;
-		default:
-			break;
-	}
-
-	QSize s( pm.size() );
-	s.scale( width() * 2 / 3, height() * 2 / 3, Qt::KeepAspectRatio );
-	p.drawImage( 5, 5, fastQImage( pm ).scaled( s ) );
-
-	QFont f = p.font();
-	f.setBold( TRUE );
-	f.setPointSize( f.pointSize() + 2 );
-	p.setFont( f );
-	p.setPen( QColor( 0, 0, 0 ) );
-	p.drawText( QRect( 5, 10 + s.height(), width() - 10,
-						height() - 15 - s.height() ),
-				Qt::TextWordWrap | Qt::AlignRight, msg );
-}
-
-
-
-
 bool client::userLoggedIn( void )
 {
 	QMutexLocker ml( &m_syncMutex );
@@ -494,9 +472,58 @@ void client::hideEvent( QHideEvent * )
 
 
 
+void client::mousePressEvent( QMouseEvent * _me )
+{
+	if( _me->button() == Qt::LeftButton )
+	{
+		raise();
+		m_clickPoint = _me->globalPos();
+		m_origPos = pos();
+	}
+	else
+	{
+		QWidget::mousePressEvent( _me );
+	}
+}
+
+
+
+
+void client::mouseMoveEvent( QMouseEvent * _me )
+{
+	if( m_clickPoint.x() >= 0 )
+	{
+		move( m_origPos + _me->globalPos() - m_clickPoint );
+		parentWidget()->updateGeometry();
+	}
+	else
+	{
+		QWidget::mouseMoveEvent( _me );
+	}
+}
+
+
+
+
+void client::mouseReleaseEvent( QMouseEvent * _me )
+{
+	m_clickPoint = QPoint( -1, -1 );
+	QWidget::mouseReleaseEvent( _me );
+}
+
+
+
+
 void client::mouseDoubleClickEvent( QMouseEvent * _me )
 {
-	remoteControl( "" );
+	if( m_mainWindow->getClassroomManager()->clientDblClickAction() == 0 )
+	{
+		remoteControl( "" );
+	}
+	else
+	{
+		viewLive( "" );
+	}
 }
 
 
@@ -504,105 +531,118 @@ void client::mouseDoubleClickEvent( QMouseEvent * _me )
 
 void client::paintEvent( QPaintEvent * _pe )
 {
-	QPainter p( this );
+	static QImage * img_unknown = NULL;
+	static QImage * img_no_user = NULL;
+	static QImage * img_host_unreachable = NULL;
+	static QImage * img_demo = NULL;
+	static QImage * img_locked = NULL;
 
-	if( m_connection->state() == ivsConnection::Connected/* && m_user != ""*/ &&
+	if( img_unknown == NULL )
+		img_unknown = new QImage( ":/resources/error.png" );
+	if( img_no_user == NULL )
+		img_no_user = new QImage( ":/resources/no_user.png" );
+	if( img_host_unreachable == NULL )
+		img_host_unreachable = new QImage( ":/resources/host_unreachable.png" );
+	if( img_demo == NULL )
+		img_demo = new QImage( ":/resources/window_demo.png" );
+	if( img_locked == NULL )
+		img_locked = new QImage( ":/resources/locked.png" );
+
+	QPainter p( this );
+	p.setBrush( Qt::white );
+	p.setPen( Qt::black );
+	p.drawRect( QRect( 0, 0, width()-1, height()-1 ) );
+	p.setRenderHint( QPainter::Antialiasing, TRUE );
+
+	const QString s = m_user != "" ? m_user :
+		( m_name + " (" + m_classRoomItem->parent()->text( 0 ) +
+									")" );
+	QFont f = p.font();
+	f.setBold( TRUE );
+	p.setFont( f );
+	p.setPen( Qt::gray );
+	p.drawText( 11, TITLE_HEIGHT-3, s );
+	p.setPen( Qt::black );
+	p.drawText( 10, TITLE_HEIGHT-4, s );
+
+	if( m_connection->state() == ivsConnection::Connected &&
 						m_mode == Mode_Overview )
 	{
-		p.drawImage( _pe->rect().topLeft(),
-				m_connection->scaledScreen(), _pe->rect() );
+		p.drawImage( CONTENT_OFFSET, m_connection->scaledScreen());
 	}
 	else
 	{
-		p.drawPixmap( 0, 0, m_statePixmap );
+	/*	p.drawImage( _pe->rect().topLeft() + CONTENT_OFFSET,
+						m_statePixmap, _pe->rect() );*/
+	const int aw = width() - 2*DECO_WIDTH;
+	const int ah = height() - CONTENT_SIZE_SUB.height() - DECO_WIDTH;
+
+	QImage * pm = img_unknown;
+	QString msg = tr( "Unknown state" );
+
+	switch( m_state )
+	{
+		case State_Overview:
+			return;
+		case State_NoUserLoggedIn:
+			pm = img_no_user;
+			msg = tr( "No user logged in" );
+			break;
+		case State_Unreachable:
+			pm = img_host_unreachable;
+			msg = tr( "Host unreachable" );
+			break;
+		case State_Demo:
+			pm = img_demo;
+			msg = tr( "Demo running" );
+			break;
+		case State_Locked:
+			pm = img_locked;
+			msg = tr( "Desktop locked" );
+			break;
+		default:
+			break;
+	}
+
+	QFont f = p.font();
+	f.setBold( TRUE );
+	f.setPointSize( f.pointSize() + 1 );
+	p.setFont( f );
+
+	QRect r = p.boundingRect( QRect( 5, 0, aw-10, 10 ),
+				Qt::TextWordWrap | Qt::AlignCenter, msg );
+	QSize s( pm->size() );
+	s.scale( aw-10, ah-r.height()-20, Qt::KeepAspectRatio );
+	p.drawImage( ( aw-s.width() ) / 2, height()-ah,
+						fastQImage( *pm ).scaled( s ) );
+
+	p.setPen( QColor( 0, 0, 0 ) );
+	p.drawText( QRect( 5, height()-r.height()-10, aw - 10,
+						r.height() ),
+				Qt::TextWordWrap | Qt::AlignCenter, msg );
 	}
 
 	if( m_makeSnapshot )
 	{
 		QMutexLocker ml( &m_syncMutex );
 		m_makeSnapshot = FALSE;
-
-		if( m_user != "" &&
-			m_connection->state() == ivsConnection::Connected )
+		if( m_connection->takeSnapshot() )
 		{
-			// construct text
-			QString txt = m_user + "@" + m_name + " (" +
-					m_localIP + ") " +
-					QDate( QDate::currentDate()
-						).toString( Qt::ISODate ) +
-					" " +
-					QTime( QTime::currentTime() ).toString(
-								Qt::ISODate );
-			const QString dir = localSystem::snapshotDir();
-			if( !localSystem::ensurePathExists( dir ) )
-			{
-				messageBox::information( tr( "Snapshot" ),
-		tr( "Could not make a snapshot as directory %1 doesn't exist "
-			"and couldn't be created." ).arg( dir ) );
-				return;
-			}
-			// construct filename
-			QString file_name =  "_" +
-						m_localIP + "_" +
-			QDate( QDate::currentDate() ).toString( Qt::ISODate ) +
-						"_" +
-		QTime( QTime::currentTime() ).toString( Qt::ISODate ) + ".png";
-			file_name.replace( ':', '-' );
-			file_name = dir + m_user.section( '(', 1, 1 ).
-					section( ')', 0, 0 ) + file_name;
-			const int FONT_SIZE = 14;
-			const int RECT_MARGIN = 10;
-			const int RECT_INNER_MARGIN = 5;
-
-			QImage screen( m_connection->screen() );
-
-			QPixmap italc_icon( QPixmap(
-					":/resources/client_observed.png" ) );
-
-			QPainter p( &screen );
-			QFont fnt = p.font();
-			fnt.setPointSize( FONT_SIZE );
-			fnt.setBold( TRUE );
-			p.setFont( fnt );
-			QFontMetrics fm( p.font() );
-
-			const int rx = RECT_MARGIN;
-			const int ry = screen.height() - RECT_MARGIN -
-					2 * RECT_INNER_MARGIN - FONT_SIZE;
-			const int rw = RECT_MARGIN + 4 * RECT_INNER_MARGIN +
-				fm.size( Qt::TextSingleLine, txt ).width() +
-							italc_icon.width();
-			const int rh = 2 * RECT_INNER_MARGIN + FONT_SIZE;
-			const int ix = rx + RECT_INNER_MARGIN;
-			const int iy = ry + RECT_INNER_MARGIN;
-			const int tx = ix + italc_icon.width() +
-							2 * RECT_INNER_MARGIN;
-			const int ty = ry + RECT_INNER_MARGIN + FONT_SIZE - 2;
-
-			p.fillRect( rx, ry, rw, rh, QColor(
-							255, 255, 255, 128 ) );
-			p.drawPixmap( ix, iy, italc_icon );
-			p.drawText( tx, ty, txt );
-			screen.save( file_name, "PNG", 50 );
-
 			s_reloadSnapshotList = TRUE;
 		}
 	}
 
-
 }
 
 
 
 
-void client::resizeEvent( QResizeEvent * )
+void client::resizeEvent( QResizeEvent * _re )
 {
-	if( findChild<QToolBar *>() )
-	{
-		findChild<QToolBar *>()->setFixedWidth( width() );
-	}
-	m_connection->setScaledSize( size() );
-	updateStatePixmap();
+	findChildren<closeButton*>()[0]->move( width()-21, 4 );
+	m_connection->setScaledSize( size() - CONTENT_SIZE_SUB );
+	m_connection->rescaleScreen();
+	QWidget::resizeEvent( _re );
 }
 
 
@@ -622,8 +662,6 @@ void client::showEvent( QShowEvent * )
 
 void client::reload( const QString & _update )
 {
-	bool available = FALSE;
-
 	if( userLoggedIn() )
 	{
 		m_syncMutex.lock();
@@ -649,28 +687,23 @@ void client::reload( const QString & _update )
 
 		m_user = m_connection->user();
 		m_syncMutex.unlock();
-
-		if( m_user != "" )
-		{
-			available = TRUE;
-			if( toolTip() != m_user )
-			{
-				setToolTip( m_user );
-			}
-		}
 	}
-
-	if( available == FALSE )
+	else
 	{
-		setToolTip( "" );
+		m_user = "";
 	}
 
 
-	// if we are called out of draw-thread, we may update...
+	// if we are called out of main-thread, we may update...
 	if( _update == CONFIRM_YES )
 	{
 		update();
 	}
+
+	if( m_classRoomItem )
+	{
+		m_classRoomItem->setUser( m_user );
+	}
 }
 
 
@@ -692,9 +725,9 @@ void client::clientDemo( const QString & )
 		}
 	}
 
-	m_mainWindow->checkModeButton( client::Mode_FullscreenDemo );
+	//m_mainWindow->checkModeButton( client::Mode_FullscreenDemo );
 
-	conn->remoteControlDisplay( m_localIP );
+	m_mainWindow->remoteControlDisplay( m_localIP, TRUE, TRUE );
 }
 
 
@@ -703,11 +736,8 @@ void client::clientDemo( const QString & )
 void client::viewLive( const QString & )
 {
 	changeMode( Mode_Overview, m_mainWindow->localISD() );
-	//m_syncMutex.lock();
 
-	m_mainWindow->localISD()->remoteControlDisplay( m_localIP, TRUE );
-
-	//m_syncMutex.unlock();
+	m_mainWindow->remoteControlDisplay( m_localIP, TRUE );
 }
 
 
@@ -716,11 +746,8 @@ void client::viewLive( const QString & )
 void client::remoteControl( const QString & )
 {
 	changeMode( Mode_Overview, m_mainWindow->localISD() );
-	//m_syncMutex.lock();
 
-	m_mainWindow->localISD()->remoteControlDisplay( m_localIP );
-
-	//m_syncMutex.unlock();
+	m_mainWindow->remoteControlDisplay( m_localIP );
 }
 
 
@@ -740,79 +767,20 @@ void client::sendTextMessage( const QString & _msg )
 	}
 	else
 	{
-		m_updateThread->enqueueCommand( updateThread::SendTextMessage,
-									_msg );
+		m_updateThread->enqueueCommand(
+				updateThread::Cmd_SendTextMessage, _msg );
 	}
 }
 
 
 
-#if 0
-void client::distributeFile( const QString & _file )
-{
-/*	QMessageBox::information( this, tr( "Function not implemented yet." ), tr( "This function is not completely implemented yet. This is why it is disabled at the moment." ), QMessageBox::Ok );
-	return;*/
-
-	if( _file == "" )
-	{
-		QFileDialog ofd( this, tr( "Select file to distribute" ),
-							QDir::homePath() );
-		ofd.setFileMode( QFileDialog::ExistingFile );
-		if( ofd.exec() == QDialog::Accepted &&
-					ofd.selectedFiles().empty() == FALSE )
-		{
-			distributeFile( ofd.selectedFiles().front() );
-		}
-	}
-	else
-	{
-		m_syncMutex.lock ();
-		m_connection->sendFile( _file );
-		m_syncMutex.unlock();
-	}
-}
-
-
-
-
-void client::collectFiles( const QString & _filter )
-{
-/*	QMessageBox::information( this, tr( "Function not implemented yet."), tr("This function is not completely implemented yet. This is why it is disabled at the moment."), QMessageBox::Ok);
-	return;*/
-
-	if( _filter.isEmpty() )
-	{
-		bool ok;
-		QString f = QInputDialog::getText( this, tr( "Collect files" ),
-						tr( "Please enter the name of "
-							"the file to be "
-							"collected.\nOnly "
-							"files located in the "
-							"PUBLIC-directory are "
-							"allowed." ),
-						QLineEdit::Normal, "", &ok );
-		if( ok && !f.isEmpty() )
-		{
-			collectFiles( f );
-		}
-	}
-	else
-	{
-		m_syncMutex.lock ();
-		m_connection->collectFiles( _filter );
-		m_syncMutex.unlock();
-	}
-}
-#endif
-
-
 
 void client::logonUser( const QString & _uname_and_pw )
 {
 
 	if( _uname_and_pw.isEmpty() )
 	{
-		multiLogonDialog mld( this );
+		remoteLogonDialog mld( this );
 		if( mld.exec() == QDialog::Accepted &&
 			!mld.userName().isEmpty() && !mld.password().isEmpty() )
 		{
@@ -822,7 +790,7 @@ void client::logonUser( const QString & _uname_and_pw )
 	}
 	else
 	{
-		m_updateThread->enqueueCommand( updateThread::LogonUserCmd,
+		m_updateThread->enqueueCommand( updateThread::Cmd_LogonUser,
 								_uname_and_pw );
 	}
 }
@@ -832,7 +800,7 @@ void client::logonUser( const QString & _uname_and_pw )
 
 void client::logoutUser( const QString & _confirm )
 {
-	m_updateThread->enqueueCommand( updateThread::LogoutUser );
+	m_updateThread->enqueueCommand( updateThread::Cmd_LogoutUser );
 }
 
 
@@ -849,30 +817,6 @@ void client::snapshot( const QString & )
 
 void client::powerOn( const QString & )
 {
-/*	QString bcast;
-	QList<QNetworkInterface> ifs = QNetworkInterface::allInterfaces();
-	for( QList<QNetworkInterface>::const_iterator it = ifs.begin();
-				it != ifs.end() && !bcast.isEmpty(); ++it )
-	{
-		QList<QNetworkAddressEntry> nae = it->addressEntries();
-		for( QList<QNetworkAddressEntry>::const_iterator it2 =
-								nae.begin();
-						it2 != nae.end(); ++it2 )
-		{
-			if( !( it2->ip() ==
-				QHostAddress( QHostAddress::LocalHost ) ) )
-			{
-				bcast = it2->broadcast().toString();
-				break;
-			}
-		}
-	}
-
-	if( bcast.isEmpty() )
-	{
-		bcast = QHostAddress( QHostAddress::Broadcast ).toString();
-	}*/
-
 	m_mainWindow->localISD()->wakeOtherComputer( m_mac );
 }
 
@@ -881,30 +825,7 @@ void client::powerOn( const QString & )
 
 void client::reboot( const QString & _confirm )
 {
-/*	if( userLoggedIn() )
-	{
-		if( QMessageBox::warning( this, tr( "User logged in" ),
-			tr( "Warning: you are trying to reboot a client at "
-				"which a user is logged in! Continue anyway?" ),
-						QMessageBox::Yes,
-						QMessageBox::No ) ==
-							QMessageBox::No )
-		{
-			return;
-		}
-	}
-	else if( _confirm == CONFIRM_YES || _confirm.isEmpty() )
-	{
-		if( QMessageBox::question( this, tr( "Reboot client" ),
-			tr( "Are you sure want to reboot selected client?" ),
-						QMessageBox::Yes,
-						QMessageBox::No ) ==
-							QMessageBox::No )
-		{
-			return;
-		}
-	}*/
-	m_updateThread->enqueueCommand( updateThread::Reboot );
+	m_updateThread->enqueueCommand( updateThread::Cmd_Reboot );
 }
 
 
@@ -913,30 +834,7 @@ void client::reboot( const QString & _confirm )
 
 void client::powerDown( const QString & _confirm )
 {
-/*	if( userLoggedIn() )
-	{
-		if( QMessageBox::warning( this, tr( "User logged in" ),
-			tr( "Warning: you are trying to power off a client at "
-				"which a user is logged in! Continue anyway?" ),
-						QMessageBox::Yes,
-						QMessageBox::No ) ==
-							QMessageBox::No )
-		{
-			return;
-		}
-	}
-	else if( _confirm == CONFIRM_YES || _confirm == "" )
-	{
-		if( QMessageBox::question( this, tr( "Power off client" ),
-			tr( "Are you sure want to power off selected client?" ),
-						QMessageBox::Yes,
-						QMessageBox::No ) ==
-							QMessageBox::No )
-		{
-			return;
-		}
-	}*/
-	m_updateThread->enqueueCommand( updateThread::PowerDown );
+	m_updateThread->enqueueCommand( updateThread::Cmd_PowerDown );
 }
 
 
@@ -957,7 +855,8 @@ void client::execCmds( const QString & _cmds )
 	}
 	else
 	{
-		m_updateThread->enqueueCommand( updateThread::ExecCmds, _cmds );
+		m_updateThread->enqueueCommand( updateThread::Cmd_ExecCmds,
+									_cmds );
 	}
 }
 
@@ -997,6 +896,14 @@ client::states client::currentState( void ) const
 
 
 
+
+
+
+
+
+
+
+
 updateThread::updateThread( client * _client ) :
 	QThread(),
 	m_client( _client )
@@ -1006,13 +913,9 @@ updateThread::updateThread( client * _client ) :
 
 
 
+
 void updateThread::update( void )
 {
-	if( mainWindow::atExit() == FALSE && m_client->isVisible() )
-	{
-		m_client->processCmd( client::Reload, CONFIRM_NO );
-	}
-
 	m_queueMutex.lock();
 	m_client->m_syncMutex.lock();
 	while( !m_queue.isEmpty() )
@@ -1021,30 +924,30 @@ void updateThread::update( void )
 		m_queueMutex.unlock();
 		switch( i.first )
 		{
-			case ResetConnection:
+			case Cmd_ResetConnection:
 				m_client->m_connection->reset(
 						i.second.toString() );
 				break;
-			case StartDemo:
+			case Cmd_StartDemo:
 				m_client->m_connection->startDemo(
 					i.second.toList()[0].toString(),
 					i.second.toList()[1].toInt() );
 				break;
-			case StopDemo:
+			case Cmd_StopDemo:
 				m_client->m_connection->stopDemo();
 				break;
-			case LockScreen:
+			case Cmd_LockScreen:
 				m_client->m_connection->lockDisplay();
 				break;
-			case UnlockScreen:
+			case Cmd_UnlockScreen:
 				m_client->m_connection->unlockDisplay();
 				break;
-			case SendTextMessage:
+			case Cmd_SendTextMessage:
 				m_client->m_connection->
 					displayTextMessage(
 						i.second.toString() );
 				break;
-			case LogonUserCmd:
+			case Cmd_LogonUser:
 			{
 				const QString s = i.second.toString();
 				const int pos = s.indexOf( '*' );
@@ -1056,18 +959,18 @@ void updateThread::update( void )
 						s.mid( pos2 + 1 ) );
 				break;
 			}
-			case LogoutUser:
+			case Cmd_LogoutUser:
 				m_client->m_connection->logoutUser();
 				break;
-			case Reboot:
+			case Cmd_Reboot:
 				m_client->m_connection->
 						restartComputer();
 				break;
-			case PowerDown:
+			case Cmd_PowerDown:
 				m_client->m_connection->
 						powerDownComputer();
 				break;
-			case ExecCmds:
+			case Cmd_ExecCmds:
 				m_client->m_connection->execCmds(
 						i.second.toString() );
 				break;
@@ -1076,9 +979,16 @@ void updateThread::update( void )
 	}
 	m_client->m_syncMutex.unlock();
 	m_queueMutex.unlock();
-	
-	if( m_client->isVisible() == FALSE &&
-		m_client->m_connection->state() == ivsConnection::Connected )
+
+	if( m_client->isVisible() )
+	{
+		if( !m_client->m_mainWindow->remoteControlRunning() &&
+						!mainWindow::atExit() )
+		{
+			m_client->processCmd( client::Cmd_Reload, CONFIRM_NO );
+		}
+	}
+	else if( m_client->m_connection->state() == ivsConnection::Connected )
 	{
 		m_client->m_connection->close();
 	}
@@ -1092,12 +1002,15 @@ void updateThread::run( void )
 	QTimer t;
 	connect( &t, SIGNAL( timeout() ), this, SLOT( update() ),
 							Qt::DirectConnection );
-	t.start( m_client->m_mainWindow->getClassroomManager()->updateInterval() *
-									1000 );
+	t.start( m_client->m_mainWindow->getClassroomManager()->updateInterval()
+								* 1000 );
 	exec();
+	update();
+	m_client->m_connection->close();
 }
 
 
 
+
 #include "client.moc"
 
